/*
 *  MooIRCd2: Next Generation Moo IRC Daemon.
 *  m_forcenick.c: forcibly sets a users nick.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_forcenick.c,v 1.2 2004/12/01 12:36:51 fwaggle Exp $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "hash.h"
#include "fdlist.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_conf.h"
#include "s_stats.h"
#include "s_user.h"
#include "hash.h"
#include "whowas.h"
#include "s_serv.h"
#include "send.h"
#include "list.h"
#include "channel.h"
#include "s_log.h"
#include "resv.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "common.h"
#include "packet.h"

static void mo_forcenick(struct Client *, struct Client *, int, char**);

static int check_clean_nick(struct Client *, struct Client *, 
                            char *, char *, char *);

static int clean_nick_name(char *);

struct Message forcenick_msgtab = {
	"FORCENICK", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_unregistered, m_not_oper, m_unregistered, mo_forcenick}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&forcenick_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&forcenick_msgtab);
}

const char *_version = "$Revision: 1.2 $";
#endif

/*
 * mo_forcenick()
 *
 *     parv[0] = sender prefix
 *     parv[1] = nickname
 */
static void
mo_forcenick(struct Client *client_p, struct Client *source_p,
             int parc, char *parv[])
{
	char     nick[NICKLEN];
	char     newnick[NICKLEN];
	struct   Client *target_p;

	/* XXX BadPtr is needed */
	if(parc < 3 || BadPtr(parv[1]) || BadPtr(parv[2]))
	{
		sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
			me.name, parv[0]);
		return;
	}

	/* mark end of grace period, to prevent nickflooding */
	if(!IsFloodDone(source_p))
		flood_endgrace(source_p);

	/* terminate nick to NICKLEN */
	strlcpy(nick, parv[1], NICKLEN);

	/* check the nickname is ok */
	if(!clean_nick_name(nick))
	{
		sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
		           me.name, parv[0], nick);
		return;
	}

	/* terminate nick to NICKLEN */
	strlcpy(newnick, parv[2], NICKLEN);

	/* check the nickname is ok */
	if(!clean_nick_name(newnick))
	{
		sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
		           me.name, parv[0], nick);
		return;
	}
  
	if ((find_client(newnick)))
	{
		sendto_one(source_p, ":%s NOTICE %s :/FORCENICK failure - new nickname (%s) in use", me.name,
		           nick, newnick);
		return;
	}

	if ((target_p = find_client(nick)))
	{
	/*
	 * If(target_p == source_p) the client is changing nicks between
	 * equivalent nicknames ie: [nick] -> {nick}
	 */

	 	if(target_p == source_p)
	 	{
	 		return;
		} else {
			change_local_nick(client_p, target_p, newnick);
			return;
		}
	} else {
		if(!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
		{
			sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name,
			           parv[0], nick);
			return;

			// NOT REACHED AT THE MOMENT
			/* The uplink might know someone by this name already. */
			sendto_one(uplink, ":%s NBURST %s %s %s", me.name, nick,
			           nick, source_p->name);

			return;
		} else {
			sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name,
			           parv[0], nick);
			return;
		}
	}
}



/* 
 * check_clean_nick()
 * 
 * input	- pointer to source
 *		- nickname
 *		- truncated nickname
 *		- origin of client
 * output	- none
 * side effects - if nickname is erroneous, or a different length to
 *                truncated nickname, return 1
 */
static int
check_clean_nick(struct Client *client_p, struct Client *source_p, 
		 char *nick, char *newnick, char *server)
{

	/* 
	 * the old code did some wacky stuff here, if the nick is invalid,
	 * kill it and dont bother messing at all
	 */

	 if(!clean_nick_name(nick) || strcmp(nick, newnick))
	 {
	 	ServerStats->is_kill++;
	 	sendto_realops_flags(FLAGS_DEBUG, L_ALL,
		                     "Bad Nick: %s From: %s(via %s)",
		                     nick, server, client_p->name);

		sendto_one(client_p, ":%s KILL %s :%s (Bad Nickname)",
		           me.name, newnick, me.name);

		/* bad nick change */
		if(source_p != client_p)
		{
			kill_client_ll_serv_butone(client_p, source_p,
                                                   "%s (Bad Nickname)",
			                           me.name);
			SetKilled(source_p);
			exit_client(client_p, source_p, &me, "Bad Nickname");
		}

		return (1);
	}

	return (0);
}

/* clean_nick_name()
 *
 * input	- nickname
 * output	- none
 * side effects - walks through the nickname, returning 0 if erroneous
 */
static int
clean_nick_name(char *nick)
{

	assert(nick);
	if(nick == NULL)
		return (0);

	/* nicks cant start with a digit or - or be 0 length */
	/* This closer duplicates behaviour of hybrid-6 */

	if (*nick == '-' || IsDigit(*nick) || *nick == '\0')
		return (0);

	for(; *nick; nick++)
	{
		if(!IsNickChar(*nick))
			return (0);
	}

	return (1);
}
